% Script to test cp4_alsls algorithm coupled with different Line Search techniques
clear all
close all
clc

%**********************************************
%--- Choose PARAMETERS of the DEMO
%**********************************************
%--- data parameters
data_type='complex';      % choose 'real' or 'complex'
I1=7;
I2=8;
I3=9;
I4=10; 
I5=11;
R=12;             % tensor rank
SNR=inf;          % SNR [dB], choose SNR=inf for a noise-free model
condA1=30;
condA2=30;
condA3=30;
condA4=30;
condA5=30;

%--- Algorithm parameters
Tol=1e-7;              % Tolerance 
MaxIt=1000;            % Max number of iterations 
lsearch='lsb';        % line search method 
Ninit=5;               % Number of random initializations 

%***************************************************
%---- Build Loading matrices and observed tensor-----
%**************************************************** 
if strcmp(data_type,'real')==1      % real initialization
   A1=randn(I1,R);
   A2=randn(I2,R);
   A3=randn(I3,R);
   A4=randn(I4,R);
   A5=randn(I5,R);
elseif strcmp(data_type,'complex')==1  % complex init
   A1=randn(I1,R)+j*randn(I1,R);
   A2=randn(I2,R)+j*randn(I2,R);
   A3=randn(I3,R)+j*randn(I3,R);
   A4=randn(I4,R)+j*randn(I4,R);
   A5=randn(I5,R)+j*randn(I5,R);
end

% Now impose a condition number on the matrices
[Ua1,Sa1,Va1]=svd(A1,0);
[Ua2,Sa2,Va2]=svd(A2,0);
[Ua3,Sa3,Va3]=svd(A3,0);
[Ua4,Sa4,Va4]=svd(A4,0);
[Ua5,Sa5,Va5]=svd(A5,0);
Sa1(1:min(I1,R),1:min(I1,R))=diag(linspace(condA1,1,min(I1,R)));   % change singular values (linearly spaced new values)
Sa2(1:min(I2,R),1:min(I2,R))=diag(linspace(condA2,1,min(I2,R)));   % change singular values (linearly spaced new values)
Sa3(1:min(I3,R),1:min(I3,R))=diag(linspace(condA3,1,min(I3,R)));   % change singular values (linearly spaced new values)
Sa4(1:min(I4,R),1:min(I4,R))=diag(linspace(condA4,1,min(I4,R)));   % change singular values (linearly spaced new values)
Sa5(1:min(I5,R),1:min(I5,R))=diag(linspace(condA5,1,min(I5,R)));   % change singular values (linearly spaced new values)
A1=Ua1*Sa1*Va1';
A2=Ua2*Sa2*Va2';
A3=Ua3*Sa3*Va3';
A4=Ua4*Sa4*Va4';
A5=Ua5*Sa5*Va5';
% create noise-free tensor
X=kr(A1,A2,A3,A4)*A5.';   % X is I1I2I3I4xI5 matrix unfolding
X=permute(reshape(X,I4,I3,I2,I1,I5),[4 3 2 1 5]);   %--> noise-free tensor
% Add noise 
if strcmp(data_type,'real')==1
   Noise_tens=randn(I1,I2,I3,I4,I5);
elseif strcmp(data_type,'complex')==1
   Noise_tens=randn(I1,I2,I3,I4,I5)+j*randn(I1,I2,I3,I4,I5);
end
sigma=(10^(-SNR/20))*(norm(X(:),'fro')/norm(Noise_tens(:),'fro'));
X=X+sigma*Noise_tens;


%-------------------------------------------------------------------------------
% COMPUTE THE CP5 decomposition of X
%-------------------------------------------------------------------------------
[A1_est,A2_est,A3_est,A4_est,A5_est,it,phi,phi_vec]=cp5_alsls(X,R,lsearch,Tol,MaxIt,Ninit);   

% compute error on loading estimates
[temp,temp,temp,errA1]=solve_perm_scale(A1_est,A1);
[temp,temp,temp,errA2]=solve_perm_scale(A2_est,A2);
[temp,temp,temp,errA3]=solve_perm_scale(A3_est,A3);
[temp,temp,temp,errA4]=solve_perm_scale(A4_est,A4);
[temp,temp,temp,errA5]=solve_perm_scale(A5_est,A5);

disp(['Final phi   :  ',num2str(phi)])
disp(['Iterations  :  ',num2str(it)])
disp(['Error on A1 :  ',num2str(errA1)])
disp(['Error on A2 :  ',num2str(errA2)])
disp(['Error on A3 :  ',num2str(errA3)])
disp(['Error on A4 :  ',num2str(errA4)])   
disp(['Error on A5 :  ',num2str(errA5)])

figure
loglog(phi_vec)
title('Evolution of cost function for the best initialization')
